__version__ = '0.4.post1'

__citation__ = """@INPROCEEDINGS{astroML,
 author={{Vanderplas}, J.T. and {Connolly}, A.J.
         and {Ivezi{\'c}}, {\v Z}. and {Gray}, A.},
 booktitle={Conference on Intelligent Data Understanding (CIDU)},
 title={Introduction to astroML: Machine learning for astrophysics},
 month={Oct.},
 pages={47 -54},
 doi={10.1109/CIDU.2012.6382200},
 year={2012}
}"""
